import { __awaiter, __decorate } from "tslib";
import { Connect } from '@genesislcap/foundation-comms';
import { ZeroAgTextRenderer } from '@genesislcap/foundation-zero-grid-pro';
import { customElement, FASTElement, observable, attr } from '@microsoft/fast-element';
import { filterObjectArray, getSelectedFields } from '../../utils';
import { FIELD_SELECT_CONFIG, SELECTED_FIELDS_CONFIG } from './columns';
import { SelectFieldsStyles } from './select-fields.styles';
import { SelectFieldsTemplate } from './select-fields.template';
let SelectFields = class SelectFields extends FASTElement {
    constructor() {
        super(...arguments);
        this.fieldsToRemove = [];
        this.report = {};
        this.selectedDatasource = '';
        this.makeSureFieldValueIsNotLost = (array) => {
            return array.map((field) => {
                var _a;
                const matchingField = (_a = this.selectedFields) === null || _a === void 0 ? void 0 : _a.find((f) => (f === null || f === void 0 ? void 0 : f.name) === (field === null || field === void 0 ? void 0 : field.name));
                return Object.assign(Object.assign({}, field), matchingField);
            });
        };
        this.onTextFieldChange = (value, params) => __awaiter(this, void 0, void 0, function* () {
            const newFields = this.selectedFields.map((field) => {
                let newField = Object.assign({}, field);
                if (field.name === params.node.data.name) {
                    newField = Object.assign(Object.assign({}, newField), { displayName: value });
                }
                return newField;
            });
            this.setSelectedFields(newFields);
        });
        this.refreshSelectFieldGridData = () => {
            const selectedFieldsOrder = [];
            this.selectedFieldsGrid.gridApi.forEachNodeAfterFilterAndSort((rowNode) => {
                selectedFieldsOrder.push(rowNode.data);
            });
            this.setSelectedFields([...this.makeSureFieldValueIsNotLost(selectedFieldsOrder)]);
        };
        this.removeSelectedFields = () => {
            const filteredFields = filterObjectArray('name', this.selectedFields, this.fieldsToRemove);
            this.setSelectedFields(filteredFields);
            this.fieldsToRemove = [];
            this.allFieldsGrid.gridApi.deselectAll();
            this.allFieldsGrid.gridApi.forEachNode((node) => {
                filteredFields.find((field) => field.name === node.data.name) && node.setSelected(true);
            });
        };
    }
    selectedFieldsChanged(oldValue) {
        if (oldValue) {
            this.selectedFieldsGrid.gridApi.setRowData(this.selectedFields);
            this.selectedFieldsGrid.gridApi.refreshCells();
        }
    }
    connectedCallback() {
        super.connectedCallback();
        this.loadAllFieldsGrid();
        this.loadSelectedFieldsGrid();
        this.allFieldsGrid.addEventListener('onGridReady', () => {
            window.addEventListener('resize', () => {
                setTimeout(() => {
                    this.allFieldsGrid.gridApi.sizeColumnsToFit();
                });
            });
            this.allFieldsGrid.gridApi.addEventListener('firstDataRendered', () => {
                this.allFieldsGrid.gridApi.sizeColumnsToFit();
                this.selectDefaultFields();
                this.allFieldsGrid.gridApi.addEventListener('selectionChanged', () => {
                    if (this.selectedFields) {
                        this.selectedFields = getSelectedFields(this.allFieldsGrid, this.selectedFields);
                        this.setSelectedFields([...this.makeSureFieldValueIsNotLost(this.selectedFields)]);
                    }
                    else {
                        this.setSelectedFields([
                            ...this.makeSureFieldValueIsNotLost(this.allFieldsGrid.gridApi.getSelectedRows()),
                        ]);
                    }
                });
            });
        });
        this.selectedFieldsGrid.addEventListener('onGridReady', () => {
            window.addEventListener('resize', () => {
                setTimeout(() => {
                    this.selectedFieldsGrid.gridApi.sizeColumnsToFit();
                });
            });
            this.selectedFieldsGrid.gridApi.addEventListener('firstDataRendered', () => {
                this.selectedFieldsGrid.gridApi.sizeColumnsToFit();
            });
            this.selectedFieldsGrid.gridApi.addEventListener('selectionChanged', () => {
                this.fieldsToRemove = this.selectedFieldsGrid.gridApi.getSelectedRows();
            });
        });
    }
    setSelectedFields(fields) {
        this.$emit('setSelectedFields', { fields });
    }
    loadAllFieldsGrid() {
        return __awaiter(this, void 0, void 0, function* () {
            this.allFieldsGrid.gridOptions = Object.assign(Object.assign({}, FIELD_SELECT_CONFIG.gridOptions), { columnDefs: FIELD_SELECT_CONFIG.colDefs, rowData: [...this.fieldData] });
        });
    }
    loadSelectedFieldsGrid() {
        return __awaiter(this, void 0, void 0, function* () {
            this.selectedFieldsGrid.gridOptions = Object.assign(Object.assign({}, SELECTED_FIELDS_CONFIG.gridOptions), { onRowDragEnd: this.refreshSelectFieldGridData, columnDefs: [
                    ...SELECTED_FIELDS_CONFIG.colDefs,
                    {
                        field: 'displayName',
                        headerName: 'Display name',
                        cellRenderer: ZeroAgTextRenderer,
                        cellRendererParams: {
                            accessor: 'displayName',
                            onChange: this.onTextFieldChange,
                        },
                    },
                    {
                        rowDrag: true,
                        width: 60,
                        sortable: false,
                        filter: false,
                    },
                ], rowData: [] });
        });
    }
    selectDefaultFields() {
        if (this.selectedFields.length) {
            this.allFieldsGrid.gridApi.forEachNode((node) => {
                this.selectedFields.find((field) => field.name === node.data.name) &&
                    node.setSelected(true);
            });
            this.setSelectedFields([...this.makeSureFieldValueIsNotLost(this.selectedFields)]);
        }
        else {
            this.selectedFields = [];
        }
    }
};
__decorate([
    Connect
], SelectFields.prototype, "connect", void 0);
__decorate([
    observable
], SelectFields.prototype, "fieldsToRemove", void 0);
__decorate([
    observable
], SelectFields.prototype, "fieldData", void 0);
__decorate([
    observable
], SelectFields.prototype, "report", void 0);
__decorate([
    observable
], SelectFields.prototype, "selectedFields", void 0);
__decorate([
    attr
], SelectFields.prototype, "selectedDatasource", void 0);
SelectFields = __decorate([
    customElement({
        name: 'reporting-select-fields',
        template: SelectFieldsTemplate,
        styles: SelectFieldsStyles,
    })
], SelectFields);
export { SelectFields };
