import { html, repeat, ref, when } from '@microsoft/fast-element';
const notificationToast = html `
  <div class="notification-container">
    ${repeat((x) => x.notifications, html `
        <zero-toast notify=${(notification) => notification.type}>
          <div slot="top">${(notification) => notification.title}</div>
          <p slot="content">${(notification) => notification.message}</p>
        </zero-toast>
      `)}
  </div>
`;
const header = html `
  <div class="header">
    <h1 id="title" class="title">All Reports</h1>
    <div class="controls">
      <zero-button
        appearance="neutral-grey"
        class="button"
        ?disabled=${(x) => !x.selectedRows.length}
        @click=${(x) => x.reportsGrid.batchDelete()}
      >
        Delete Reports
      </zero-button>

      <zero-button appearance="primary-gradient" class="button" @click=${(x) => x.create()}>
        Add new report
      </zero-button>
    </div>
  </div>
`;
export const ReportingHomeTemplate = html `
  ${when((x) => x.notifications.length, notificationToast)}

  <div class="container">
    <zero-design-system-provider>
      <zero-tabs ${ref('tabs')} class="tabs">
        <span part="start" class="start" slot="start">Reporting</span>
        <zero-tab id="list-reports" slot="tab">All Reports</zero-tab>
        <zero-tab id="create-edit-report" slot="tab">Create Report</zero-tab>
        ${when((x) => Object.keys(x.activeReport).length > 0, html `
            <zero-tab id="run-report" slot="tab">Run Report</zero-tab>
          `)}

        <zero-tab-panel slot="tabpanel" class="tab-panel">
          ${header}
          <reporting-list ${ref('reportsGrid')}></reporting-list>
        </zero-tab-panel>

        <zero-tab-panel slot="tabpanel" class="tab-panel">
          <reporting-create-edit
            :report=${(x) => x.activeReport}
            :setPageTab=${(x) => x.setPageTab}
            :datasourceSettings=${(x) => x.datasourceSettings}
          />
        </zero-tab-panel>

        ${when((x) => Object.keys(x.activeReport).length > 0, html `
            <zero-tab-panel slot="tabpanel" class="tab-panel">
              <reporting-preview
                :report=${(x) => x.activeReport}
                :datasourceSettings=${(x) => x.datasourceSettings}
              />
            </zero-tab-panel>
          `)}
      </zero-tabs>
    </zero-design-system-provider>
  </div>
`;
